@@ -1,16 +1,16 @@
 from os.path import exists, expanduser
 from typing import Optional, Union
 
-from pytezos.context.impl import ExecutionContext  # type: ignore
+from pytezos.context.impl import DEFAULT_IPFS_GATEWAY, ExecutionContext  # type: ignore
 from pytezos.crypto.encoding import is_pkh, is_public_key
 from pytezos.crypto.key import Key, is_installed
 from pytezos.jupyter import InlineDocstring
 from pytezos.rpc import RpcMultiNode, RpcNode, ShellQuery
 from pytezos.rpc.errors import RpcError
 
-default_network = 'edo2net'
+default_network = 'florencenet'
 default_key = 'edsk33N474hxzA4sKeWVM6iuGNGDpX2mGwHNxEA4UbWS8sW3Ta3NKH'  # please, use responsibly
-default_key_hash = 'tz1cnQZXoznhduu4MVWfJF6GSyP6mMHMbbWa'
+default_key_hash = 'tz1grSQDByRpnVs7sPtaprNZRp531ZKz6Jmm'
 
 alice_key = 'edsk3QoqBuvdamxouPhin7swCvkQNgq4jP5KZPbwWNnwdZpSpJiEbq'  # for flextesa sandbox
 alice_key_hash = 'tz1VSUr8wwNhLAzempoch5d6hLRiTh8Cjcjb'
@@ -19,14 +19,12 @@ dictator_key = 'edsk31vznjHSSpGExDMHYASz45VZqXN4DPxvsa4hAyY8dHM28cZzp6'  # for t
 
 nodes = {
     'mainnet': ['https://mainnet-tezos.giganode.io/',
-                'https://rpc.tzkt.io/mainnet/',
                 'https://api.tez.ie/',
-                'https://tezos-prod.cryptonomic-infra.tech/chains/main/blocks/head'],
+                'https://tezos-prod.cryptonomic-infra.tech/'],
     'sandbox': ['http://127.0.0.1:8732/'],
     'localhost': ['http://127.0.0.1:8732/'],
-    'delphinet': ['https://rpc.tzkt.io/delphinet/'],
-    'edonet': ['https://rpc.tzkt.io/edonet/'],
-    'edo2net': ['https://rpc.tzkt.io/edo2net/']
+    'florencenet': ['https://florence-tezos.giganode.io/'],
+    'edo2net': ['https://edonet-tezos.giganode.io/']
 }
 keys = {
     'alice': alice_key,
@@ -115,13 +113,16 @@ class ContextMixin(metaclass=InlineDocstring):
             res.append(f'.block_id\t{self.block_id}')
         return '\n'.join(res)
 
-    def _spawn_context(self,
-                       shell: Optional[Union[ShellQuery, str]] = None,
-                       key: Optional[Union[Key, str, dict]] = None,
-                       address: Optional[str] = None,
-                       block_id: Optional[Union[str, int]] = None,
-                       mode: Optional[str] = None,
-                       script: Optional[dict] = None) -> ExecutionContext:
+    def _spawn_context(
+        self,
+        shell: Optional[Union[ShellQuery, str]] = None,
+        key: Optional[Union[Key, str, dict]] = None,
+        address: Optional[str] = None,
+        block_id: Optional[Union[str, int]] = None,
+        mode: Optional[str] = None,
+        script: Optional[dict] = None,
+        ipfs_gateway: Optional[str] = None,
+    ) -> ExecutionContext:
         if isinstance(shell, str):
             if shell.endswith('.pool'):
                 shell = shell.split('.')[0]
@@ -162,4 +163,6 @@ class ContextMixin(metaclass=InlineDocstring):
             address=address,
             block_id=block_id,
             script=script or self.context.script,
-            mode=mode or self.context.mode)
+            mode=mode or self.context.mode,
+            ipfs_gateway=ipfs_gateway,
+        )
