@@ -4,6 +4,7 @@ from typing import Callable, Dict, List, Optional, Sequence, Tuple, Type, TypeVa
 
 from typing_extensions import Literal
 
+from pytezos.logging import logger
 from pytezos.michelson.forge import unforge_address, unforge_chain_id, unforge_micheline, unforge_public_key, unforge_signature
 from pytezos.michelson.format import micheline_to_michelson
 
@@ -173,6 +174,15 @@ class Micheline(metaclass=ErrorTrace):
         elif isinstance(expr, dict):
             if expr.get('prim'):
                 prim, args, annots = parse_micheline_prim(expr)
+
+                # FIXME: We need entrypoint to be an argument
+                if prim == 'RUN':
+                    if annots:
+                        args = [{'string': annots[0][1:]}] + args  # type: ignore
+                        annots = []
+                    else:
+                        args = [{'string': 'default'}] + args  # type: ignore
+
                 args_len = len(args)
                 if (prim, args_len) not in Micheline.classes:
                     args_len = None  # type: ignore
