@@ -14,9 +14,9 @@ from pytezos.operation import DEFAULT_BURN_RESERVE, DEFAULT_GAS_RESERVE, MAX_OPE
 from pytezos.operation.content import ContentMixin
 from pytezos.operation.fees import calculate_fee, default_fee, default_gas_limit, default_storage_limit
 from pytezos.operation.forge import forge_operation_group
-from pytezos.operation.kind import validation_passes
 from pytezos.operation.result import OperationResult
 from pytezos.rpc.errors import RpcError
+from pytezos.rpc.kind import validation_passes
 
 
 class OperationGroup(ContextMixin, ContentMixin):
@@ -32,6 +32,8 @@ class OperationGroup(ContextMixin, ContentMixin):
         chain_id: Optional[int] = None,
         branch: Optional[str] = None,
         signature: Optional[str] = None,
+        opg_hash: Optional[str] = None,
+        # TODO: metadata {balance_updates, operation_result}
     ):
         super().__init__(context=context)
         self.contents = contents or []
@@ -39,6 +41,7 @@ class OperationGroup(ContextMixin, ContentMixin):
         self.chain_id = chain_id
         self.branch = branch
         self.signature = signature
+        self.opg_hash = opg_hash
 
     def __repr__(self):
         res = [
@@ -58,6 +61,7 @@ class OperationGroup(ContextMixin, ContentMixin):
             chain_id=kwargs.get('chain_id', self.chain_id),
             branch=kwargs.get('branch', self.branch),
             signature=kwargs.get('signature', self.signature),
+            opg_hash=kwargs.get('opg_hash', self.opg_hash)
         )
 
     def json_payload(self) -> dict:
@@ -142,7 +146,7 @@ class OperationGroup(ContextMixin, ContentMixin):
             branch=branch,
         )
 
-    def run(self, block_id='head'):
+    def run(self, block_id: str = 'head'):
         """Simulate operation without signature checks.
 
         :param block_id: Specify a level at which this operation should be applied (default is head)
@@ -187,7 +191,7 @@ class OperationGroup(ContextMixin, ContentMixin):
         if len(self.contents) != 1 or self.contents[0]['kind'] != 'failing_noop':
             raise NotImplementedError('Use for signing messages only')
 
-        branch = block if is_bh(block) else self.shell.blocks[block].hash()
+        branch = block if is_bh(str(block)) else self.shell.blocks[block].hash()
         return b'\x03' + bytes.fromhex(self._spawn(branch=branch).forge())
 
     def autofill(
@@ -284,9 +288,13 @@ class OperationGroup(ContextMixin, ContentMixin):
         hash_digest = blake2b_32(self.binary_payload()).digest()
         return base58_encode(hash_digest, b'o').decode()
 
-    def run_operation(self):
-        # TODO: Docstring
-        return self.shell.blocks['head'].helpers.scripts.run_operation.post(self.json_payload())
+    def run_operation(self, block_id: str = 'head'):
+        """Simulate operation without signature checks.
+
+        :param block_id: Specify a level at which this operation should be applied (default is head)
+        :returns: RPC response from `run_operation`
+        """
+        return self.run(block_id)
 
     @deprecated(deprecated_in='3.1.0', removed_in='4.0.0', details='use `run_operation()` instead')
     def preapply(self):
@@ -299,6 +307,26 @@ class OperationGroup(ContextMixin, ContentMixin):
 
         return self.run_operation()
 
+    def send(self,
+             gas_reserve: int = DEFAULT_GAS_RESERVE,
+             burn_reserve: int = DEFAULT_BURN_RESERVE,
+             min_confirmations: int = 0,
+             ttl: Optional[int] = None) -> 'OperationGroup':
+        """
+
+        :param gas_reserve: Add a safe reserve for dynamically calculated gas limit (default is 100).
+        :param burn_reserve: Add a safe reserve for dynamically calculated storage limit (default is 100).
+        :param min_confirmations: number of block injections to wait for before returning (default is 0, i.e. async mode)
+        :param ttl: Number of blocks to wait in the mempool before removal (default is 5 for public network, 60 for sandbox)
+        :return: OperationGroup with hash filled
+        """
+        if ttl is None:
+            ttl = self.context.get_operations_ttl()
+
+        opg = self.autofill(gas_reserve=gas_reserve, burn_reserve=burn_reserve, ttl=ttl).sign()
+        res = opg.inject(min_confirmations=min_confirmations, num_blocks_wait=ttl)
+        return opg._spawn(opg_hash=res['hash'])
+
     def inject(
         self,
         check_result: bool = True,
@@ -315,10 +343,6 @@ class OperationGroup(ContextMixin, ContentMixin):
         :param min_confirmations: number of block injections to wait for before returning
         :returns: operation group with metadata (raw RPC response)
         """
-        if kwargs.get('_async'):
-            logger.warning('`_async` argument is deprecated, use `min_confirmations` instead')
-            min_confirmations = 0 if kwargs['_async'] is True else 1
-
         self.context.reset()
 
         opg_hash = self.shell.injection.operation.post(
